@@ -286,7 +286,6 @@ class PhaseTDNewStatistic(NewSNRStatistic):
         self.param_bin = {}
         self.two_det_flag = (len(ifos) == 2)
         self.two_det_weights = {}
-        self.pb_int_size = None
 
     def get_hist(self, ifos=None):
         """Read in a signal density file for the ifo combination"""
@@ -376,17 +375,23 @@ class PhaseTDNewStatistic(NewSNRStatistic):
                 # "weights" table a O(N) rather than O(NlogN) operation. It
                 # sacrifices RAM to do this, so is a good tradeoff for 2
                 # detectors, but not for 3!
-                pb_iinfo = numpy.iinfo(self.param_bin[ifo]['c0'].dtype)
-                self.pb_int_size = pb_iinfo.max - pb_iinfo.min + 1
+                if not hasattr(self, 'c0_size'):
+                    self.c0_size = {}
+                    self.c1_size = {}
+                    self.c2_size = {}
 
-                array_size = [self.pb_int_size, self.pb_int_size,
-                              self.pb_int_size]
+                self.c0_size[ifo] = 2 * (abs(self.param_bin[ifo]['c0']).max() + 1)
+                self.c1_size[ifo] = 2 * (abs(self.param_bin[ifo]['c1']).max() + 1)
+                self.c2_size[ifo] = 2 * (abs(self.param_bin[ifo]['c2']).max() + 1)
+
+                array_size = [self.c0_size[ifo], self.c1_size[ifo],
+                              self.c2_size[ifo]]
                 dtypec = self.weights[ifo].dtype
                 self.two_det_weights[ifo] = \
                     numpy.zeros(array_size, dtype=dtypec) + self.max_penalty
-                id0 = self.param_bin[ifo]['c0'] + self.pb_int_size // 2
-                id1 = self.param_bin[ifo]['c1'] + self.pb_int_size // 2
-                id2 = self.param_bin[ifo]['c2'] + self.pb_int_size // 2
+                id0 = self.param_bin[ifo]['c0'].astype(numpy.int32) + self.c0_size[ifo] // 2
+                id1 = self.param_bin[ifo]['c1'].astype(numpy.int32) + self.c1_size[ifo] // 2
+                id2 = self.param_bin[ifo]['c2'].astype(numpy.int32) + self.c2_size[ifo] // 2
                 self.two_det_weights[ifo][id0, id1, id2] = self.weights[ifo]
 
         relfac = histfile.attrs['sensitivity_ratios']
@@ -500,10 +505,20 @@ class PhaseTDNewStatistic(NewSNRStatistic):
             # Read signal weight from precalculated histogram
             if self.two_det_flag:
                 # High-RAM, low-CPU option for two-det
-                id0 = nbinned['c0'] + self.pb_int_size // 2
-                id1 = nbinned['c1'] + self.pb_int_size // 2
-                id2 = nbinned['c2'] + self.pb_int_size // 2
-                rate[rtype] = self.two_det_weights[ref_ifo][id0, id1, id2]
+                rate[rtype] = numpy.zeros(len(nbinned)) + self.max_penalty
+
+                id0 = nbinned['c0'].astype(numpy.int32) + self.c0_size[ref_ifo] // 2
+                id1 = nbinned['c1'].astype(numpy.int32) + self.c1_size[ref_ifo] // 2
+                id2 = nbinned['c2'].astype(numpy.int32) + self.c2_size[ref_ifo] // 2
+
+                # look up keys which are within boundaries
+                within = (id0 > 0) & (id0 < self.c0_size[ref_ifo])
+                within = within & (id1 > 0) & (id1 < self.c1_size[ref_ifo])
+                within = within & (id2 > 0) & (id2 < self.c2_size[ref_ifo])
+                within = numpy.where(within)[0]
+
+                rate[rtype[within]] = self.two_det_weights[ref_ifo][id0[within], id1[within], id2[within]]
+
             else:
                 # Low[er]-RAM, high[er]-CPU option for >two det
                 loc = numpy.searchsorted(self.param_bin[ref_ifo], nbinned)
@@ -515,7 +530,7 @@ class PhaseTDNewStatistic(NewSNRStatistic):
                 missed = numpy.where(
                     self.param_bin[ref_ifo][loc] != nbinned
                 )[0]
-                rate[rtype][missed] = self.max_penalty
+                rate[rtype[missed]] = self.max_penalty
 
             # Scale by signal population SNR
             rate[rtype] *= (sref / self.ref_snr) ** -4.0
@@ -1192,6 +1207,24 @@ class ExpFitSGFgBgNormNewStatistic(PhaseTDNewStatistic,
         self.fits_by_tid[ifo]['median_sigma'] = \
             coeff_file['median_sigma'][:][tid_sort]
 
+    def lognoiserate(self, trigs, alphabelow=6):
+        """Calculate the log noise rate density over single-ifo newsnr
+
+        Read in single trigger information, make the newsnr statistic
+        and rescale by the fitted coefficients alpha and rate
+        """
+        alphai, ratei, thresh = self.find_fits(trigs)
+        newsnr = self.get_newsnr(trigs)
+        # Above the threshold we use the usual fit coefficient (alpha)
+        # below threshold use specified alphabelow
+        bt = newsnr < thresh
+        lognoisel = - alphai * (newsnr - thresh) + numpy.log(alphai) + \
+                        numpy.log(ratei)
+        lognoiselbt = - alphabelow * (newsnr - thresh) + \
+                           numpy.log(alphabelow) + numpy.log(ratei)
+        lognoisel[bt] = lognoiselbt[bt]
+        return numpy.array(lognoisel, ndmin=1, dtype=numpy.float32)
+
     def single(self, trigs):
         # single-ifo stat = log of noise rate
         sngl_stat = self.lognoiserate(trigs)
@@ -1280,6 +1313,13 @@ class ExpFitSGFgBgNormNewStatistic(PhaseTDNewStatistic,
 
 
 class ExpFitSGPSDFgBgNormStatistic(ExpFitSGFgBgNormNewStatistic):
+    def __init__(self, files=None, ifos=None, **kwargs):
+        ExpFitSGFgBgNormNewStatistic.__init__(self, files=files, ifos=ifos,
+                                              **kwargs)
+        self.get_newsnr = ranking.get_newsnr_sgveto_psdvar
+
+
+class ExpFitSGPSDScaledFgBgNormStatistic(ExpFitSGFgBgNormNewStatistic):
     def __init__(self, files=None, ifos=None, **kwargs):
         ExpFitSGFgBgNormNewStatistic.__init__(self, files=files, ifos=ifos,
                                               **kwargs)
@@ -1290,7 +1330,7 @@ class ExpFitSGPSDFgBgNormBBHStatistic(ExpFitSGFgBgNormNewStatistic):
     def __init__(self, files=None, ifos=None, max_chirp_mass=None, **kwargs):
         ExpFitSGFgBgNormNewStatistic.__init__(self, files=files, ifos=ifos,
                                               **kwargs)
-        self.get_newsnr = ranking.get_newsnr_sgveto_psdvar_scaled_threshold
+        self.get_newsnr = ranking.get_newsnr_sgveto_psdvar
         self.mcm = max_chirp_mass
         self.curr_mchirp = None
 
@@ -1312,6 +1352,13 @@ class ExpFitSGPSDFgBgNormBBHStatistic(ExpFitSGFgBgNormNewStatistic):
         return logr_s
 
 
+class ExpFitSGPSDSTFgBgNormBBHStatistic(ExpFitSGPSDFgBgNormBBHStatistic):
+    def __init__(self, files=None, ifos=None, max_chirp_mass=None, **kwargs):
+        ExpFitSGPSDFgBgNormBBHStatistic.__init__(self, files=files, ifos=ifos,
+                                                 max_chirp_mass=None, **kwargs)
+        self.get_newsnr = ranking.get_newsnr_sgveto_psdvar_scaled_threshold
+
+
 statistic_dict = {
     'newsnr': NewSNRStatistic,
     'network_snr': NetworkSNRStatistic,
@@ -1334,8 +1381,8 @@ statistic_dict = {
     'exp_fit_sg_bg_rate': ExpFitSGBgRateStatistic,
     'exp_fit_sg_fgbg_rate': ExpFitSGFgBgRateStatistic,
     'exp_fit_sg_fgbg_norm_new': ExpFitSGFgBgNormNewStatistic,
-    '2ogc': ExpFitSGPSDFgBgNormStatistic, # backwards compatible
-    '2ogcbbh': ExpFitSGPSDFgBgNormBBHStatistic, # backwards compatible
+    '2ogc': ExpFitSGPSDScaledFgBgNormStatistic, # backwards compatible
+    '2ogcbbh': ExpFitSGPSDSTFgBgNormBBHStatistic, # backwards compatible
     'exp_fit_sg_fgbg_norm_psdvar': ExpFitSGPSDFgBgNormStatistic,
     'exp_fit_sg_fgbg_norm_psdvar_bbh': ExpFitSGPSDFgBgNormBBHStatistic
 }
