@@ -136,7 +136,7 @@ class SingleCoincForGraceDB(object):
 
         proc_id = ligolw_process.register_to_xmldoc(
             outdoc, 'pycbc', {}, ifos=usable_ifos, comment='',
-            version=pycbc_version.git_hash,
+            version=pycbc_version.version,
             cvs_repository='pycbc/'+pycbc_version.git_branch,
             cvs_entry_time=pycbc_version.date).process_id
 
@@ -289,7 +289,7 @@ class SingleCoincForGraceDB(object):
             logging.info('Source probabilities file saved as %s', prob_fname)
 
     def upload(self, fname, gracedb_server=None, testing=True,
-               extra_strings=None):
+               extra_strings=None, search='AllSky'):
         """Upload this trigger to gracedb
 
         Parameters
@@ -302,6 +302,8 @@ class SingleCoincForGraceDB(object):
         testing: bool
             Switch to determine if the upload should be sent to gracedb as a
             test trigger (True) or a production trigger (False).
+        search: str
+            String going into the "search" field of the GraceDB event.
         """
         from ligo.gracedb.rest import GraceDb
         import matplotlib
@@ -380,7 +382,7 @@ class SingleCoincForGraceDB(object):
 
             # create GraceDB event
             group = 'Test' if testing else 'CBC'
-            r = gracedb.createEvent(group, "pycbc", fname, "AllSky").json()
+            r = gracedb.createEvent(group, "pycbc", fname, search).json()
             gid = r["graceid"]
             logging.info("Uploaded event %s", gid)
 
@@ -397,9 +399,13 @@ class SingleCoincForGraceDB(object):
                              psd_fname, open(fname, "rb").read(), "psd")
             logging.info("Uploaded PSDs for event %s", gid)
 
-            # add other tags and comments
-            gracedb.writeLog(
-                    gid, "Using PyCBC code hash %s" % pycbc_version.git_hash)
+            # add info for tracking code version
+            version_str = 'Using PyCBC version {}{} at {}'
+            version_str = version_str.format(
+                    pycbc_version.version,
+                    ' (release)' if pycbc_version.release else '',
+                    os.path.dirname(pycbc.__file__))
+            gracedb.writeLog(gid, version_str)
 
             extra_strings = [] if extra_strings is None else extra_strings
             for text in extra_strings:
