@@ -490,10 +490,11 @@ class LiveFilterBank(TemplateBank):
         super(LiveFilterBank, self).__init__(filename, approximant=approximant,
                 parameters=parameters, **kwds)
         self.ensure_standard_filter_columns(low_frequency_cutoff=low_frequency_cutoff)
-        self.hash_lookup = {}
+        self.param_lookup = {}
         for i, p in enumerate(self.table):
-            hash_value =  hash((p.mass1, p.mass2, p.spin1z, p.spin2z))
-            self.hash_lookup[hash_value] = i
+            key =  (p.mass1, p.mass2, p.spin1z, p.spin2z)
+            assert(key not in self.param_lookup) # Uh, oh, template confusion!
+            self.param_lookup[key] = i
 
     def round_up(self, num):
         """Determine the length to use for this waveform by rounding.
@@ -519,20 +520,20 @@ class LiveFilterBank(TemplateBank):
         instance.table = self.table[sindex]
         return instance
 
-    def id_from_hash(self, hash_value):
-        """Get the index of this template based on its hash value
+    def id_from_param(self, param_tuple):
+        """Get the index of this template based on its param tuple
 
         Parameters
         ----------
-        hash : int
-            Value of the template hash
+        param_tuple : tuple
+            Tuple of the parameters which uniquely identify this template
 
         Returns
         --------
         index : int
             The ordered index that this template has in the template bank.
         """
-        return self.hash_lookup[hash_value]
+        return self.param_lookup[param_tuple]
 
     def __getitem__(self, index):
         if isinstance(index, slice):
@@ -600,10 +601,10 @@ class LiveFilterBank(TemplateBank):
         # Add sigmasq as a method of this instance
         htilde.sigmasq = types.MethodType(sigma_cached, htilde)
 
-        htilde.id = self.id_from_hash(hash((htilde.params.mass1,
-                                      htilde.params.mass2,
-                                      htilde.params.spin1z,
-                                      htilde.params.spin2z)))
+        htilde.id = self.id_from_param((htilde.params.mass1,
+                                        htilde.params.mass2,
+                                        htilde.params.spin1z,
+                                        htilde.params.spin2z))
         return htilde
 
 
